@@ -1,34 +1,39 @@
 //
-// Copyright (c) 2006-2012, Benjamin Kaufmann
+// Copyright (c) 2006-2017 Benjamin Kaufmann
 //
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/
 //
-// Clasp is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
 //
-// Clasp is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
 //
-// You should have received a copy of the GNU General Public License
-// along with Clasp; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+// IN THE SOFTWARE.
 //
 #include "clasp_app.h"
 #include <clasp/solver.h>
 #include <clasp/dependency_graph.h>
 #include <clasp/parser.h>
-#include <potassco/aspif.h>
 #include <clasp/clause.h>
+#include <potassco/aspif.h>
+#include <potassco/string_convert.h>
 #include <iostream>
 #include <fstream>
 #include <cstdlib>
 #include <climits>
+#include <signal.h>
 #ifdef _WIN32
-#define snprintf _snprintf
 #pragma warning (disable : 4996)
 #endif
 #if defined(__GLIBC__) || defined(__GNU_LIBRARY__)
@@ -66,31 +71,31 @@ namespace Cli {
 ClaspAppOptions::ClaspAppOptions() : outf(0), compute(0), ifs(' '), hideAux(false), onlyPre(0), printPort(false) {
 	quiet[0] = quiet[1] = quiet[2] = static_cast<uint8>(UCHAR_MAX);
 }
-void ClaspAppOptions::initOptions(ProgramOptions::OptionContext& root) {
-	using namespace ProgramOptions;
+void ClaspAppOptions::initOptions(Potassco::ProgramOptions::OptionContext& root) {
+	using namespace Potassco::ProgramOptions;
 	OptionGroup basic("Basic Options");
 	basic.addOptions()
+		("print-portfolio,@1", flag(printPort), "Print default portfolio and exit")
 		("quiet,q"    , notify(this, &ClaspAppOptions::mappedOpts)->implicit("2,2,2")->arg("<levels>"),
 		 "Configure printing of models, costs, and calls\n"
 		 "      %A: <mod>[,<cost>][,<call>]\n"
 		 "        <mod> : print {0=all|1=last|2=no} models\n"
-		 "        <cost>: print {0=all|1=last|2=no} optimize values [<m>]\n"
+		 "        <cost>: print {0=all|1=last|2=no} optimize values [<mod>]\n"
 		 "        <call>: print {0=all|1=last|2=no} call steps      [2]")
-		("pre", notify(this, &ClaspAppOptions::mappedOpts)->arg("<fmt>")->implicit("1"), "Print simplified program and exit\n"
-		 "      %A: Output format: {1=auto|2=aspif}")
-		("print-portfolio" , flag(printPort), "Print default portfolio and exit")
+		("pre", notify(this, &ClaspAppOptions::mappedOpts)->arg("<fmt>")->implicit("aspif"), "Print simplified program and exit\n"
+		 "      %A: Set output format to {aspif|smodels} (implicit: %I)")
 		("outf,@1", storeTo(outf)->arg("<n>"), "Use {0=default|1=competition|2=JSON|3=no} output")
-		("out-atomf,@1" , storeTo(outAtom), "Set atom format string (<Pre>?%%0<Post>?)")
-		("out-ifs,@1"   , notify(this, &ClaspAppOptions::mappedOpts), "Set internal field separator")
+		("out-atomf,@2" , storeTo(outAtom), "Set atom format string (<Pre>?%%0<Post>?)")
+		("out-ifs,@2"   , notify(this, &ClaspAppOptions::mappedOpts), "Set internal field separator")
 		("out-hide-aux,@1" , flag(hideAux), "Hide auxiliary atoms in answers")
 		("lemma-in,@1"     , storeTo(lemmaIn)->arg("<file>"), "Read additional lemmas from %A")
 		("lemma-out,@1"    , storeTo(lemmaLog)->arg("<file>"), "Log learnt lemmas to %A")
-		("lemma-out-lbd,@1", storeTo(lemma.lbdMax)->arg("<n>"), "Only log lemmas with lbd <= %A")
-		("lemma-out-max,@1", storeTo(lemma.logMax)->arg("<n>"), "Stop logging after %A lemmas")
-		("lemma-out-dom,@1", notify(this, &ClaspAppOptions::mappedOpts), "Log lemmas over <arg {input|output}> variables")
-		("lemma-out-txt,@1", flag(lemma.logText), "Log lemmas as ground integrity constraints")
-		("hcc-out,@1", storeTo(hccOut)->arg("<file>"), "Write non-hcf programs to %A.#scc")
-		("file,f,@2" , storeTo(input)->composing(), "Input files")
+		("lemma-out-lbd,@2", storeTo(lemma.lbdMax)->arg("<n>"), "Only log lemmas with lbd <= %A")
+		("lemma-out-max,@2", storeTo(lemma.logMax)->arg("<n>"), "Stop logging after %A lemmas")
+		("lemma-out-dom,@2", notify(this, &ClaspAppOptions::mappedOpts), "Log lemmas over <arg {input|output}> variables")
+		("lemma-out-txt,@2", flag(lemma.logText), "Log lemmas as ground integrity constraints")
+		("hcc-out,@2", storeTo(hccOut)->arg("<file>"), "Write non-hcf programs to %A.#scc")
+		("file,f,@3" , storeTo(input)->composing(), "Input files")
 		("compute,@2", storeTo(compute)->arg("<lit>"), "Force given literal to true")
 	;
 	root.add(basic);
@@ -99,7 +104,7 @@ bool ClaspAppOptions::mappedOpts(ClaspAppOptions* this_, const std::string& name
 	if (name == "quiet") {
 		const char* err = 0;
 		uint32      q[3]= {uint32(UCHAR_MAX),uint32(UCHAR_MAX),uint32(UCHAR_MAX)};
-		int      parsed = bk_lib::xconvert(value.c_str(), q, &err);
+		int      parsed = Potassco::xconvert(value.c_str(), q, &err);
 		for (int i = 0; i != parsed; ++i) { this_->quiet[i] = static_cast<uint8>(q[i]); }
 		return parsed && *err == 0;
 	}
@@ -115,11 +120,12 @@ bool ClaspAppOptions::mappedOpts(ClaspAppOptions* this_, const std::string& name
 		return (this_->lemma.domOut = (strcasecmp(value.c_str(), "output") == 0)) == true || strcasecmp(value.c_str(), "input") == 0;
 	}
 	else if (name == "pre") {
-		return value.size() == 1 && (this_->onlyPre = static_cast<uint8>(Potassco::BufferedStream::toDigit(value[0]))) <= 2;
+		if      (strcasecmp(value.c_str(), "aspif")   == 0) { this_->onlyPre = (int8)AspParser::format_aspif; return true; }
+		else if (strcasecmp(value.c_str(), "smodels") == 0) { this_->onlyPre = (int8)AspParser::format_smodels; return true; }
 	}
 	return false;
 }
-bool ClaspAppOptions::validateOptions(const ProgramOptions::ParsedOptions&) {
+bool ClaspAppOptions::validateOptions(const Potassco::ProgramOptions::ParsedOptions&) {
 	if (quiet[1] == static_cast<uint8>(UCHAR_MAX)) { quiet[1] = quiet[0]; }
 	return true;
 }
@@ -139,45 +145,34 @@ const int* ClaspAppBase::getSignals() const {
 }
 bool ClaspAppBase::parsePositional(const std::string& t, std::string& out) {
 	int num;
-	if   (bk_lib::string_cast(t, num)) { out = "number"; }
+	if (Potassco::string_cast(t, num)) { out = "number"; }
 	else                               { out = "file";   }
 	return true;
 }
-void ClaspAppBase::initOptions(ProgramOptions::OptionContext& root) {
+void ClaspAppBase::initOptions(Potassco::ProgramOptions::OptionContext& root) {
 	claspConfig_.addOptions(root);
 	claspAppOpts_.initOptions(root);
 	root.find("verbose")->get()->value()->defaultsTo("1");
 }
 
-void ClaspAppBase::validateOptions(const ProgramOptions::OptionContext&, const ProgramOptions::ParsedOptions& parsed, const ProgramOptions::ParsedValues& values) {
+void ClaspAppBase::validateOptions(const Potassco::ProgramOptions::OptionContext&, const Potassco::ProgramOptions::ParsedOptions& parsed, const Potassco::ProgramOptions::ParsedValues& values) {
 	if (claspAppOpts_.printPort) {
 		printTemplate();
 		exit(E_UNKNOWN);
 	}
 	setExitCode(E_NO_RUN);
-	using ProgramOptions::Error;
 	ProblemType pt = getProblemType();
-	if (!claspAppOpts_.validateOptions(parsed) || !claspConfig_.finalize(parsed, pt, true)) {
-		throw Error("command-line error!");
-	}
+	POTASSCO_REQUIRE(claspAppOpts_.validateOptions(parsed) && claspConfig_.finalize(parsed, pt, true), "command-line error!");
 	ClaspAppOptions& app = claspAppOpts_;
-	if (!app.lemmaLog.empty() && !isStdOut(app.lemmaLog)) {
-		if (std::find(app.input.begin(), app.input.end(), app.lemmaLog) != app.input.end() || app.lemmaIn == app.lemmaLog) {
-			throw Error("'lemma-out': cowardly refusing to overwrite input file!");
-		}
-	}
-	if (!app.lemmaIn.empty() && !isStdIn(app.lemmaIn) && !std::ifstream(app.lemmaIn.c_str()).is_open()) {
-		error("'lemma-in': could not open file!");
-		exit(E_NO_RUN);
-	}
+	POTASSCO_REQUIRE(app.lemmaLog.empty() || isStdOut(app.lemmaLog) || (std::find(app.input.begin(), app.input.end(), app.lemmaLog) == app.input.end() && app.lemmaIn != app.lemmaLog),
+		"'lemma-out': cowardly refusing to overwrite input file!");
+	POTASSCO_REQUIRE(app.lemmaIn.empty() || isStdIn(app.lemmaIn) || std::ifstream(app.lemmaIn.c_str()).is_open(),
+		"'lemma-in': could not open file!");
 	for (std::size_t i = 1; i < app.input.size(); ++i) {
-		if (!isStdIn(app.input[i]) && !std::ifstream(app.input[i].c_str()).is_open()) {
-			throw Error(ClaspStringBuffer().appendFormat("'%s': could not open input file!", app.input[i].c_str()).c_str());
-		}
-	}
-	if (app.onlyPre && pt != Problem_t::Asp) {
-		throw Error("Option '--pre' only supported for ASP!");
+		POTASSCO_EXPECT(isStdIn(app.input[i]) || std::ifstream(app.input[i].c_str()).is_open(),
+			"'%s': could not open input file!", app.input[i].c_str());
 	}
+	POTASSCO_REQUIRE(!app.onlyPre || pt == Problem_t::Asp, "Option '--pre' only supported for ASP!");
 	setExitCode(0);
 	storeCommandArgs(values);
 }
@@ -207,8 +202,7 @@ void ClaspAppBase::shutdown() {
 	const ClaspFacade::Summary& result = clasp_->shutdown();
 	if (shutdownTime_g) {
 		shutdownTime_g += RealTime::getTime();
-		char msg[80];
-		info(clasp_format(msg, sizeof(msg), "Shutdown completed in %.3f seconds", shutdownTime_g));
+		info(POTASSCO_FORMAT("Shutdown completed in %.3f seconds", shutdownTime_g));
 	}
 	if (out_.get())  { out_->shutdown(result); }
 	setExitCode(getExitCode() | exitCode(result));
@@ -316,62 +310,67 @@ void ClaspAppBase::printTemplate() const {
 		printf("%s: %s\n", it.name(), it.args());
 	}
 }
-
 void ClaspAppBase::printVersion() {
-	ProgramOptions::Application::printVersion();
+	Potassco::Application::printVersion();
 	printLibClaspVersion();
+	printLicense();
+}
+void ClaspAppBase::printLicense() const {
+	printf("License: The MIT License <https://opensource.org/licenses/MIT>\n");
 }
-
 void ClaspAppBase::printLibClaspVersion() const {
-	if (strcmp(getName(), "clasp") != 0) {
-		printf("libclasp version %s\n", CLASP_VERSION);
-	}
-	printf("Configuration: WITH_THREADS=%d", WITH_THREADS);
-#if defined(WITH_THREADS) && defined(TBB_VERSION_MAJOR) && WITH_THREADS
-	printf(" (Intel TBB version %d.%d)", TBB_VERSION_MAJOR, TBB_VERSION_MINOR);
-#endif
-	printf("\n%s\n", CLASP_LEGAL);
+	printf("libclasp version %s (libpotassco version %s)\n", CLASP_VERSION, LIB_POTASSCO_VERSION);
+	printf("Configuration: WITH_THREADS=%d\n", CLASP_HAS_THREADS);
+	printf("%s\n", CLASP_LEGAL);
 	fflush(stdout);
 }
 
-void ClaspAppBase::printHelp(const ProgramOptions::OptionContext& root) {
-	ProgramOptions::Application::printHelp(root);
-	if (root.getActiveDescLevel() >= ProgramOptions::desc_level_e1) {
+void ClaspAppBase::printHelp(const Potassco::ProgramOptions::OptionContext& root) {
+	Potassco::Application::printHelp(root);
+	if (root.getActiveDescLevel() >= Potassco::ProgramOptions::desc_level_e1) {
 		printf("[asp] %s\n", ClaspCliConfig::getDefaults(Problem_t::Asp));
 		printf("[cnf] %s\n", ClaspCliConfig::getDefaults(Problem_t::Sat));
 		printf("[opb] %s\n", ClaspCliConfig::getDefaults(Problem_t::Pb));
 	}
-	if (root.getActiveDescLevel() >= ProgramOptions::desc_level_e2) {
+	if (root.getActiveDescLevel() >= Potassco::ProgramOptions::desc_level_e2) {
 		printf("\nDefault configurations:\n");
 		printDefaultConfigs();
 	}
+	else {
+		const char* ht3 = "\nType ";
+		if (root.getActiveDescLevel() == Potassco::ProgramOptions::desc_level_default) {
+			printf("\nType '%s --help=2' for more options and defaults\n", getName());
+			ht3 = "and ";
+		}
+		printf("%s '%s --help=3' for all options and configurations.\n", ht3, getName());
+	}
 	fflush(stdout);
 }
-
-void ClaspAppBase::printDefaultConfigs() const {
+void ClaspAppBase::printConfig(ConfigKey k) const {
 	uint32 minW = 2, maxW = 80;
-	std::string cmd;
+	ConfigIter it = ClaspCliConfig::getConfig(k);
+	printf("%s:\n%*c", it.name(), minW-1, ' ');
+	const char* opts = it.args();
+	for (std::size_t size = std::strlen(opts), n = maxW - minW; n < size;) {
+		while (n && opts[n] != ' ') { --n; }
+		if (!n) { break; }
+		printf("%.*s\n%*c", static_cast<int>(n), opts, static_cast<int>(minW - 1), ' ');
+		size -= n + 1;
+		opts += n + 1;
+		n = (maxW - minW);
+	}
+	printf("%s\n", opts);
+}
+void ClaspAppBase::printDefaultConfigs() const {
 	for (int i = Clasp::Cli::config_default+1; i != Clasp::Cli::config_default_max_value; ++i) {
-		ConfigIter it = ClaspCliConfig::getConfig(static_cast<Clasp::Cli::ConfigKey>(i));
-		printf("%s:\n%*c", it.name(), minW-1, ' ');
-		cmd = it.args();
-		// split options into formatted lines
-		std::size_t sz = cmd.size(), off = 0, n = maxW - minW;
-		while (n < sz) {
-			while (n != off  && cmd[n] != ' ') { --n; }
-			if (n != off) { cmd[n] = 0; printf("%s\n%*c", &cmd[off], minW-1, ' '); }
-			else          { break; }
-			off = n+1;
-			n   = (maxW - minW) + off;
-		}
-		printf("%s\n", cmd.c_str()+off);
+		printConfig(static_cast<Clasp::Cli::ConfigKey>(i));
 	}
 }
 void ClaspAppBase::writeNonHcfs(const PrgDepGraph& graph) const {
-	char buf[10];
+	Potassco::StringBuilder buf;
 	for (PrgDepGraph::NonHcfIter it = graph.nonHcfBegin(), end = graph.nonHcfEnd(); it != end; ++it) {
-		snprintf(buf, 10, ".%u", (*it)->id());
-		WriteCnf cnf(claspAppOpts_.hccOut + buf);
+		buf.appendFormat(".%u", (*it)->id());
+		WriteCnf cnf(claspAppOpts_.hccOut + buf.c_str());
 		const SharedContext& ctx = (*it)->ctx();
 		cnf.writeHeader(ctx.numVars(), ctx.numConstraints());
 		cnf.write(ctx.numVars(), ctx.shortImplications());
@@ -383,6 +382,7 @@ void ClaspAppBase::writeNonHcfs(const PrgDepGraph& graph) const {
 			cnf.write(ctx.master()->trail()[i]);
 		}
 		cnf.close();
+		buf.clear();
 	}
 }
 std::istream& ClaspAppBase::getStream(bool reopen) const {
@@ -393,9 +393,7 @@ std::istream& ClaspAppBase::getStream(bool reopen) const {
 		isOpen = true;
 		if (!claspAppOpts_.input.empty() && !isStdIn(claspAppOpts_.input[0])) {
 			file.open(claspAppOpts_.input[0].c_str());
-			if (!file.is_open()) {
-				throw std::runtime_error(ClaspStringBuffer().appendFormat("Can not read from '%s'", claspAppOpts_.input[0].c_str()).c_str());
-			}
+			POTASSCO_EXPECT(file.is_open(), "Can not read from '%s'!", claspAppOpts_.input[0].c_str());
 		}
 	}
 	return file.is_open() ? file : std::cin;
@@ -415,7 +413,7 @@ Output* ClaspAppBase::createOutput(ProblemType f) {
 			outFormat = TextOutput::format_aspcomp;
 		}
 		out.reset(new TextOutput(verbose(), outFormat, claspAppOpts_.outAtom.c_str(), claspAppOpts_.ifs));
-		if (claspConfig_.solve.maxSat && f == Problem_t::Sat) {
+		if (claspConfig_.parse.isEnabled(ParserOptions::parse_maxsat) && f == Problem_t::Sat) {
 			static_cast<TextOutput*>(out.get())->result[TextOutput::res_sat] = "UNKNOWN";
 		}
 	}
@@ -436,7 +434,7 @@ Output* ClaspAppBase::createOutput(ProblemType f) {
 	}
 	return out.release();
 }
-void ClaspAppBase::storeCommandArgs(const ProgramOptions::ParsedValues&) {
+void ClaspAppBase::storeCommandArgs(const Potassco::ProgramOptions::ParsedValues&) {
 	/* We don't need the values */
 }
 void ClaspAppBase::handleStartOptions(ClaspFacade& clasp) {
@@ -453,7 +451,7 @@ void ClaspAppBase::handleStartOptions(ClaspFacade& clasp) {
 			typedef Potassco::AbstractProgram PrgAdapter;
 			LemmaIn(const std::string& fn, PrgAdapter* prg) : Potassco::AspifInput(*prg), prg_(prg) {
 				if (!isStdIn(fn)) { file_.open(fn.c_str()); }
-				CLASP_FAIL_IF(!accept(getStream()), "'lemma-in': invalid input file");
+				POTASSCO_REQUIRE(accept(getStream()), "'lemma-in': invalid input file!");
 			}
 			~LemmaIn() { delete prg_; }
 		private:
@@ -476,7 +474,18 @@ bool ClaspAppBase::handlePostGroundOptions(ProgramBuilder& prg) {
 	prg.endProgram();
 	if (prg.type() == Problem_t::Asp) {
 		Asp::LogicProgram& asp = static_cast<Asp::LogicProgram&>(prg);
-		AspParser::Format outf = claspAppOpts_.onlyPre == 2 || !asp.supportsSmodels() ? AspParser::format_aspif : AspParser::format_smodels;
+		AspParser::Format outf = static_cast<AspParser::Format>(claspAppOpts_.onlyPre);
+		if (outf == AspParser::format_smodels && !asp.supportsSmodels()) {
+			std::ofstream null;
+			try { AspParser::write(asp, null, outf); }
+			catch (const std::logic_error& e) {
+				error("Option '--pre': unsupported input format!");
+				info(std::string(e.what()).append(" in 'smodels' format").c_str());
+				info("Try '--pre=aspif' to print in 'aspif' format");
+				setExitCode(E_ERROR);
+				return false;
+			}
+		}
 		AspParser::write(asp, std::cout, outf);
 	}
 	else {
@@ -512,10 +521,10 @@ void ClaspApp::run(ClaspFacade& clasp) {
 	ClaspAppBase::run(clasp);
 }
 
-void ClaspApp::printHelp(const ProgramOptions::OptionContext& root) {
+void ClaspApp::printHelp(const Potassco::ProgramOptions::OptionContext& root) {
 	ClaspAppBase::printHelp(root);
-	printf("\nclasp is part of Potassco: %s\n", "https://potassco.org/clasp/");
-	printf("Get help/report bugs via : %s\n", "https://potassco.org/support/");
+	printf("\nclasp is part of Potassco: %s\n", "http://potassco.org/clasp");
+	printf("Get help/report bugs via : %s\n"  , "http://potassco.org/support\n");
 	fflush(stdout);
 }
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -526,7 +535,7 @@ LemmaLogger::LemmaLogger(const std::string& to, const Options& o)
 	, inputType_(Problem_t::Asp)
 	, options_(o)
 	, step_(0) {
-	CLASP_FAIL_IF(!str_, "Could not open lemma log file '%s'!", to.c_str());
+	POTASSCO_EXPECT(str_, "Could not open lemma log file '%s'!", to.c_str());
 }
 LemmaLogger::~LemmaLogger() { close(); }
 void LemmaLogger::startStep(ProgramBuilder& prg, bool inc) {
@@ -572,13 +581,14 @@ void LemmaLogger::add(const Solver& s, const LitVec& cc, const ConstraintInfo& i
 		if (!s.resolveToFlagged(cc, vf, temp, lbd) || lbd > options_.lbdMax) { return; }
 		out = &temp;
 	}
-	ClaspStringBuffer lemma;
-	if (options_.logText) { formatText(*out, s.sharedContext()->output, lbd, lemma); }
-	else                  { formatAspif(*out, lbd, lemma); }
-	fwrite(lemma.c_str(), sizeof(char), lemma.size(), str_);
+	char buffer[1024];
+	Potassco::StringBuilder str(buffer, sizeof(buffer), Potassco::StringBuilder::Dynamic);
+	if (options_.logText) { formatText(*out, s.sharedContext()->output, lbd, str); }
+	else                  { formatAspif(*out, lbd, str); }
+	fwrite(str.c_str(), sizeof(char), str.size(), str_);
 	++logged_;
 }
-void LemmaLogger::formatAspif(const LitVec& cc, uint32, ClaspStringBuffer& out) const {
+void LemmaLogger::formatAspif(const LitVec& cc, uint32, Potassco::StringBuilder& out) const {
 	out.appendFormat("1 0 0 0 %u", (uint32)cc.size());
 	for (LitVec::const_iterator it = cc.begin(), end = cc.end(); it != end; ++it) {
 		Literal sLit = ~*it; // clause -> constraint
@@ -592,7 +602,7 @@ void LemmaLogger::formatAspif(const LitVec& cc, uint32, ClaspStringBuffer& out)
 	}
 	out.append("\n");
 }
-void LemmaLogger::formatText(const LitVec& cc, const OutputTable& tab, uint32 lbd, ClaspStringBuffer& out) const {
+void LemmaLogger::formatText(const LitVec& cc, const OutputTable& tab, uint32 lbd, Potassco::StringBuilder& out) const {
 	out.append(":-");
 	const char* sep = " ";
 	for (LitVec::const_iterator it = cc.begin(), end = cc.end(); it != end; ++it) {
@@ -628,7 +638,7 @@ void LemmaLogger::close() {
 // WriteCnf
 /////////////////////////////////////////////////////////////////////////////////////////
 WriteCnf::WriteCnf(const std::string& outFile) : str_(fopen(outFile.c_str(), "w")) {
-	CLASP_FAIL_IF(!str_, "Could not open cnf file '%s'!", outFile.c_str());
+	POTASSCO_EXPECT(str_, "Could not open cnf file '%s'!", outFile.c_str());
 }
 WriteCnf::~WriteCnf() { close(); }
 void WriteCnf::writeHeader(uint32 numVars, uint32 numCons) {
